package geniusweb.protocol.session.mopac;

import static org.junit.Assert.assertEquals;

import java.io.IOException;
import java.net.URISyntaxException;
import java.util.Arrays;
import java.util.Collections;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;

import org.junit.Test;

import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.ObjectMapper;

import geniusweb.actions.PartyId;
import geniusweb.inform.Agreements;
import geniusweb.protocol.ProtocolException;
import tudelft.utilities.junit.GeneralTests;

public class PartyStatesTest extends GeneralTests<PartyStates> {
	private final ObjectMapper jackson = new ObjectMapper();

	private final PartyId party1 = new PartyId("party1");
	private final PartyId party2 = new PartyId("party2");
	private final PartyId party3 = new PartyId("party3");

	private final Map<PartyId, Integer> powers = new HashMap<>();
	private final PartyStates states1, states1a, states2;
	private final String serialized = "{\"powers\":{\"party2\":3,\"party1\":2,\"party3\":3},\"notYetActed\":[\"party2\",\"party1\",\"party3\"],\"actions\":[],\"agreements\":{},\"walkedAway\":[],\"exceptions\":{}}";

	public PartyStatesTest() {

		// FIXME use the other constructor to really test this.
//		PartyStates(Set<PartyId> notYetActed, List<Action> actions,
//				Agreements agreements, List<PartyId> walkedAway,
//				Map<PartyId, ProtocolException> exceptions,
//				Map<PartyId, Integer> powers)
//		
		powers.put(party1, 2);
		powers.put(party2, 3);
		states2 = new PartyStates(powers);
		powers.put(party3, 3);
		states1 = new PartyStates(powers);
		states1a = new PartyStates(powers);
	}

	@Override
	public List<List<PartyStates>> getGeneralTestData() {
		return Arrays.asList(Arrays.asList(states1, states1a),
				Arrays.asList(states2));
	}

	@Override
	public List<String> getGeneralTestStrings() {
		return Arrays.asList(
				"PartyStates.*\\[party2, party1, party3\\],\\[\\],Agreements\\{\\},\\[\\],\\{\\}.*",
				"PartyStates.*\\[party2, party1\\],\\[\\],Agreements\\{\\},\\[\\],\\{\\}.*");
	}

	@Test
	public void testBasics() {
		assertEquals(powers.keySet(), states1.getNegotiatingParties());
		assertEquals(powers.keySet(), states1.getNotYetActed());
		assertEquals(0, states1.getExceptions().size());
	}

	@Test
	public void testException() {
		PartyStates newstates = states1
				.with(new ProtocolException("bla", party1));
		assertEquals(new Agreements(), newstates.getAgreements());
		assertEquals(powers.keySet(), states1.getNegotiatingParties());
		assertEquals(new HashSet<>(Arrays.asList(party2, party3)),
				newstates.getNotYetActed());
	}

	@Test
	public void testFinish() {
		PartyStates newstates = states1.finish();
		assertEquals(new Agreements(), newstates.getAgreements());
		assertEquals(powers.keySet(), states1.getNegotiatingParties());
		assertEquals(Collections.emptySet(), newstates.getNotYetActed());
	}

	@Test
	public void testDeserialize() throws IOException {
		PartyStates obj = jackson.readValue(serialized, PartyStates.class);
		System.out.println(obj);
		assertEquals(states1, obj);
	}

	@Test
	public void testSerialize()
			throws JsonProcessingException, URISyntaxException {

		String string = jackson.writeValueAsString(states1);
		System.out.println(string);
		assertEquals(serialized, string);
	}

	@Test
	public void testWalkAway() {
		PartyStates walkawaystate = states1.withWalkAway(party2);
		assertEquals(new HashSet<>(Arrays.asList(party1, party3)),
				walkawaystate.getNotYetActed());
		assertEquals(Arrays.asList(party2), walkawaystate.getWalkedAway());
	}
}
